#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/pkg.h"

#include <ini.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

size_t lm_pkg_data_depend_count(lm_pkg_t *pkg){
  size_t index = 0;

  if(NULL == pkg->depends)
    return 0;

  while(pkg->depends[index] != NULL)
    index++;

  return index;
}

bool lm_pkg_data_depend_add(lm_pkg_t *pkg, char *depend){
  if(NULL == depend){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(!is_pkg_name_valid(depend)){
    lm_error_set(LM_ERR_PkgBadName);
    return false;
  }

  if(NULL == pkg->depends){
    pkg->depends = malloc(sizeof(char*)*2);
    pkg->depends[0] = strdup(depend);
    pkg->depends[1] = NULL;
    return true;
  }

  size_t count = lm_pkg_data_depend_count(pkg);
  pkg->depends = realloc(pkg->depends, sizeof(char*)*(count+2));
  pkg->depends[count++] = strdup(depend);
  pkg->depends[count] = NULL;
  return true;
}

int lm_pkg_data_handler(void *data, const char *_section, const char *_key, const char *_value) {
  char      *section = (char *)_section, *value = (char *)_value, *key = (char *)_key;
  lm_pkg_t *pkg = data;

  if(NULL == pkg->name){
    if(!is_pkg_name_valid(section))
      return 0;
    pkg->name = strdup(section);
  }

  else if(!eq(pkg->name, section))
    return 0;

  if(eq(key, PKG_DATA_DESC))
    pkg->desc = strdup(value);
  
  else if(eq(key, PKG_DATA_VERSION))
    pkg->version = strdup(value);
  
  else if(eq(key, PKG_DATA_SIZE))
    pkg->size = atol(value);
  
  else if(eq(key, PKG_DATA_DEPENDS)){
    if(!lm_pkg_data_depend_add(pkg, value))
      return 0;
  }

  return 1;
}

bool lm_pkg_data_load(lm_pkg_t *pkg, char *file){
  lm_pkg_data_free(pkg);

  if (ini_parse(file, lm_pkg_data_handler, pkg) < 0) {
    lm_error_set(LM_ERR_PkgDataBad);
    return false;
  }

  return true;
}

void lm_pkg_data_free(lm_pkg_t *pkg){
  free(pkg->desc);
  free(pkg->name);
  free(pkg->version);

  if(NULL != pkg->depends){
    for(int i = 0; pkg->depends[i] != NULL; i++)
      free(pkg->depends[i]);
  }
}
